@@ -27,8 +27,10 @@
 #ifndef _API_STRUCTURESH_
 #define _API_STRUCTURESH_
 
+#include <wchar.h>
 #include "api-types.h"
 #include "structures.h" /* for definition of Link */
+#include "corpus/corpus.h"
 #include "error.h"
 
 typedef enum
@@ -64,17 +66,20 @@ struct Parse_Options_s
 	int min_null_count;    /* The minimum number of null links to allow */
 	int max_null_count;    /* The maximum number of null links to allow */
 	int null_block;        /* consecutive blocks of this many words are
-			    considered as one null link  (default=1) */
+	                          considered as one null link  (default=1) */
 	int islands_ok;        /* If TRUE, then linkages with islands
-			    (separate component of the link graph)
-			    will be generated (default=FALSE) */
+	                          (separate component of the link graph)
+	                          will be generated (default=FALSE) */
 	int twopass_length;    /* min length for two-pass post processing */
 	int max_sentence_length;
 	int short_length;      /* Links that are limited in length can be
-			    no longer than this.  Default = 6 */
+	                        * no longer than this.  Default = 6 */
 	int all_short;         /* If true, there can be no connectors that are exempt */
+	int use_spell_guess;   /* Perform spell-guessing of unknown words. */ 
 	Cost_Model cost_model; /* For sorting linkages in post_processing */
-	Resources resources;   /* For deciding when to "abort" the parsing */
+	Resources resources;   /* For deciding when to abort the parsing */
+
+	/* Flags governing the command-line client; not used by parser */
 	int display_short;
 	int display_word_subscripts;  /* as in "dog.n" as opposed to "dog" */
 	int display_link_subscripts;  /* as in "Ss" as opposed to "S" */
@@ -104,22 +109,33 @@ typedef struct
 struct Dictionary_s
 {
 	Dict_node *     root;
+	Regex_node *    regex_root;
 	const char *    name;
+	const char *    lang;
 
 	int             use_unknown_word;
 	int             unknown_word_defined;
+
+	/* If not null, then use spelling guesser for unknown words */
+	void *          spell_checker; /* spell checker handle */
+#if USE_CORPUS
+	Corpus *        corpus; /* Statistics database */
+#endif
+
+#if DONT_USE_REGEX_GUESSING
+	/* English language morphology bits
+	 * replaced by regex-based morpho guesser
+	 * Dead code, remove at leisure.
+	 */
 	int             capitalized_word_defined;
 	int             pl_capitalized_word_defined;
 	int             hyphenated_word_defined;
 	int             number_word_defined;
-
-	/* English language morphology bits
-	 * XXX should be replaced by language-indep morphology.
-	 */
 	int             ing_word_defined;
 	int             s_word_defined;
 	int             ed_word_defined;
 	int             ly_word_defined;
+#endif /* DONT_USE_REGEX_GUESSING */
 
 	int             left_wall_defined;
 	int             right_wall_defined;
@@ -152,8 +168,8 @@ struct Dictionary_s
 	 */
 	Exp *           exp_list;
 
-	/* Private data elements that come in play only while distionary
-	 * is being read. 
+	/* Private data elements that come in play only while the
+	 * dictionary is being read, and are not otherwise used.
 	 */
 	FILE *          fp;
 	char            token[MAX_TOKEN_LENGTH];
@@ -161,6 +177,7 @@ struct Dictionary_s
 	wint_t          already_got_it;
 	int             line_number;
 	int             recursive_error;   /* boolean */
+	mbstate_t       mbss; /* multi-byte shift state */
 };
 
 struct Label_node_s
