@@ -4,6 +4,8 @@
  * Data for corpus statistics, used to provide a parse ranking
  * to drive the SAT solver, as well as parse ranking with the
  * ordinary solver. 
+ *
+ * Copyright (c) 2008, 2009 Linas Vepstas <linasvepstas@gmail.com>
  */
 
 #include <stdio.h>
@@ -14,30 +16,64 @@
 #include "../api-structures.h"
 #include "../utilities.h"
 
+struct corpus_s
+{
+	char * dbname;
+	sqlite3 *dbconn;
+	sqlite3_stmt *rank_query;
+	sqlite3_stmt *sense_query;
+	char *errmsg;
+};
+
+struct sense_s
+{
+	int word;
+	const char * inflected_word;
+	const char * disjunct;
+	char * sense;
+	double score;
+	Sense *next;
+};
+
 /* ========================================================= */
 
+static void * db_file_open(const char * dbname, void * user_data)
+{
+	Corpus *c = (Corpus *) user_data;
+	int rc;
+	sqlite3 *dbconn;
+	rc = sqlite3_open(dbname, &dbconn);
+	if (rc)
+	{
+		sqlite3_close(dbconn);
+		return NULL;
+	}
+
+	c->dbname = strdup(dbname);
+	return dbconn;
+}
+
+
 /**
  * Initialize the corpus statistics subsystem.
  */
 Corpus * lg_corpus_new(void)
 {
-	char * dbname;
 	int rc;
 
 	Corpus *c = (Corpus *) malloc(sizeof(Corpus));
 	c->rank_query = NULL;
 	c->sense_query = NULL;
 	c->errmsg = NULL;
+	c->dbname = NULL;
 
-	dbname = "/home/linas/src/novamente/src/link-grammar/trunk/data/sql/disjuncts.db";
+	/* dbname = "/link-grammar/data/en/sql/disjuncts.db"; */
 
-	rc = sqlite3_open(dbname, &c->dbconn);
-	if (rc)
+	c->dbconn = object_open("sql/disjuncts.db", db_file_open, c);
+	if (NULL == c->dbconn)
 	{
 		prt_error("Warning: Can't open database: %s\n",
 			sqlite3_errmsg(c->dbconn));
-		sqlite3_close(c->dbconn);
-		c->dbconn = NULL;
 		return c;
 	}
 
@@ -52,9 +88,12 @@ Corpus * lg_corpus_new(void)
 			sqlite3_errmsg(c->dbconn));
 	}
 
+	/* Results are returned in sorted order .. would it be faster
+	 * to sort locally? Don't know ... */
 	rc = sqlite3_prepare_v2(c->dbconn, 	
 		"SELECT word_sense, log_cond_probability FROM DisjunctSenses "
-		"WHERE inflected_word = ? AND disjunct = ?;",
+		"WHERE inflected_word = ? AND disjunct = ? "
+		"ORDER BY log_cond_probability DESC;",
 		-1, &c->sense_query, NULL);
 	if (rc != SQLITE_OK)
 	{
@@ -62,7 +101,7 @@ Corpus * lg_corpus_new(void)
 			sqlite3_errmsg(c->dbconn));
 	}
 
-	prt_error("Info: linkgrammar corpus statistics database opened\n");
+	prt_error("Info: Corpus statistics database found at %s\n", c->dbname);
 	return c;
 }
 
@@ -71,6 +110,8 @@ Corpus * lg_corpus_new(void)
  */ 
 void lg_corpus_delete(Corpus *c)
 {
+	if (NULL == c) return;
+
 	if (c->rank_query)
 	{
 		sqlite3_finalize(c->rank_query);
@@ -88,6 +129,12 @@ void lg_corpus_delete(Corpus *c)
 		sqlite3_close(c->dbconn);
 		c->dbconn = NULL;
 	}
+
+	if (c->dbname)
+	{
+		free(c->dbname);
+		c->dbname = NULL;
+	}
 	free(c);
 }
 
@@ -160,64 +207,6 @@ static double get_disjunct_score(Corpus *corp,
 
 /* ========================================================= */
 
-/**
- * lg_corpus_senses -- Given word and disjunct, look up senses.
- *
- * Given a particular disjunct for a word, look up its most
- * likely sense assignments from the database. 
- *
- * XXX this function might be removed in the future; it is 
- * conceptually cleaner to implement in a library sitting above
- * link-grammar. It does not currently provide any info that 
- * can be used to steer the link-grammar parsing process.
- */
-void lg_corpus_senses(Corpus *corp,
-                      const char * inflected_word,
-                      const char * disjunct)
-{
-	double log_prob;
-	const unsigned char *sense;
-	int rc;
-
-	/* Look up the disjunct in the database */
-	rc = sqlite3_bind_text(corp->sense_query, 1,
-		inflected_word, -1, SQLITE_STATIC);
-	if (rc != SQLITE_OK)
-	{
-		prt_error("Error: SQLite can't bind word in sense query: rc=%d \n", rc);
-		return;
-	}
-
-	rc = sqlite3_bind_text(corp->sense_query, 2,
-		disjunct, -1, SQLITE_STATIC);
-	if (rc != SQLITE_OK)
-	{
-		prt_error("Error: SQLite can't bind disjunct in sense query: rc=%d \n", rc);
-		return;
-	}
-
-	rc = sqlite3_step(corp->sense_query);
-	if (rc != SQLITE_ROW)
-	{
-		// printf ("Word=%s dj=%s not found in sense dict\n",
-		//	inflected_word, disjunct);
-	}
-	else
-	{
-		sense = sqlite3_column_text(corp->sense_query, 0);
-		log_prob = sqlite3_column_double(corp->sense_query, 1);
-		printf ("Word=%s dj=%s sense=%s score=%f\n", 
-			inflected_word, disjunct, sense, log_prob);
-	}
-
-	/* Failure to do both a reset *and* a clear will cause subsequent
-	 * binds tp fail. */
-	sqlite3_reset(corp->sense_query);
-	sqlite3_clear_bindings(corp->sense_query);
-}
-
-/* ========================================================= */
-
 /**
  * lg_corpus_score -- compute parse-ranking score for sentence.
  *
@@ -231,10 +220,11 @@ void lg_corpus_senses(Corpus *corp,
  * probability p(d|w) of observing disjunct 'd', given word 'w'.
  * Lower scores are better -- they indicate more likely parses.
  */
-void lg_corpus_score(Corpus *corp, Sentence sent, Linkage_info *lifo)
+void lg_corpus_score(Sentence sent, Linkage_info *lifo)
 {
 	const char *infword, *djstr;
 	double tot_score = 0.0f;
+	Corpus *corp = sent->dict->corpus;
 	int nwords = sent->length;
 	int w;
 
@@ -268,3 +258,172 @@ void lg_corpus_score(Corpus *corp, Sentence sent, Linkage_info *lifo)
 	tot_score /= nwords;
 	lifo->corpus_cost = tot_score;
 }
+
+/* ========================================================= */
+
+/**
+ * lg_corpus_senses -- Given word and disjunct, look up senses.
+ *
+ * Given a particular disjunct for a word, look up its most
+ * likely sense assignments from the database. 
+ */
+
+static Sense * lg_corpus_senses(Corpus *corp,
+                                const char * inflected_word,
+                                const char * disjunct)
+{
+	double log_prob;
+	const unsigned char *sense;
+	Sense *sns, *head = NULL;
+	int rc;
+
+	/* Look up the disjunct in the database */
+	rc = sqlite3_bind_text(corp->sense_query, 1,
+		inflected_word, -1, SQLITE_STATIC);
+	if (rc != SQLITE_OK)
+	{
+		prt_error("Error: SQLite can't bind word in sense query: rc=%d \n", rc);
+		return NULL;
+	}
+
+	rc = sqlite3_bind_text(corp->sense_query, 2,
+		disjunct, -1, SQLITE_STATIC);
+	if (rc != SQLITE_OK)
+	{
+		prt_error("Error: SQLite can't bind disjunct in sense query: rc=%d \n", rc);
+		return NULL;
+	}
+
+	rc = sqlite3_step(corp->sense_query);
+	while (SQLITE_ROW == rc)
+	{
+		sense = sqlite3_column_text(corp->sense_query, 0);
+		log_prob = sqlite3_column_double(corp->sense_query, 1);
+		// printf ("Word=%s dj=%s sense=%s score=%f\n", 
+		// 	inflected_word, disjunct, sense, log_prob);
+
+		sns = (Sense *) malloc(sizeof(Sense));
+		sns->next = head;
+		head = sns;
+
+		sns->inflected_word = inflected_word;
+		sns->disjunct = disjunct;
+		sns->sense = strdup(sense);
+		sns->score = log_prob;
+
+		/* Get the next row, if any */
+		rc = sqlite3_step(corp->sense_query);
+	}
+
+	/* Failure to do both a reset *and* a clear will cause subsequent
+	 * binds tp fail. */
+	sqlite3_reset(corp->sense_query);
+	sqlite3_clear_bindings(corp->sense_query);
+
+	return head;
+}
+
+/* ========================================================= */
+
+/**
+ * lg_corpus_linkage_senses -- Given a linkage, look up senses.
+ *
+ * Given a particular linakge, look up the most likely sense
+ * assignments from the database. 
+ *
+ * This function is not used to guide the parsing process; it is
+ * only an informational look-up.
+ */
+
+Sense * lg_corpus_linkage_senses(Linkage linkage)
+{
+	const char * infword;
+	Sentence sent = linkage->sent;
+	Dictionary dict = sent->dict; 
+	Corpus *corp = dict->corpus;
+	int nwords = sent->length;
+	Linkage_info *lifo = linkage->info;
+	int w;
+	Sense *head = NULL;
+	Sense *sns, *first, *last;
+
+	/* Decrement nwords, so as to ignore the RIGHT-WALL */
+	nwords --;
+
+	/* Loop over each word in the sentence (skipping LEFT-WALL, which is
+	 * word 0. */
+	// for (w=1; w<nwords; w++)
+	for (w=nwords-1; 0<w; w--)
+	{
+		/* If the word is not inflected, then sent->word[w].d is NULL */
+		if (sent->word[w].d)
+			infword = sent->word[w].d->string;
+		else
+			infword = sent->word[w].string;
+
+		first = lg_corpus_senses(corp, infword, lifo->disjunct_list_str[w]);
+		if (first)
+		{
+			last = first;
+			while (last->next)
+			{
+				last->word = w;
+				last = last->next;
+			}
+			last->word = w;
+
+			last->next = head;
+			head = first;
+		}
+	}
+
+	return head;
+}
+
+/* ========================================================= */
+/* Return bits and pieces of the sense assignments */
+
+Sense * lg_sense_next(Sense *sns)
+{
+	return sns->next;
+}
+
+int lg_sense_get_index(Sense *sns)
+{
+	return sns->word;
+}
+
+const char * lg_sense_get_subscripted_word(Sense *sns)
+{
+	return sns->inflected_word;
+}
+
+const char * lg_sense_get_disjunct(Sense *sns)
+{
+	return sns->disjunct;
+}
+
+const char * lg_sense_get_sense(Sense *sns)
+{
+	return sns->sense;
+}
+
+double lg_sense_get_score(Sense *sns)
+{
+	return sns->score;
+}
+
+void lg_sense_delete(Sense *sns)
+{
+	while (sns)
+	{
+		Sense * nxt = sns->next;
+		free(sns->sense);
+		free(sns);
+		sns = nxt;
+	}
+}
+
+/* ======================= END OF FILE ===================== */
+
+
